using System.Net.Http.Headers;
using System.Net.Http.Json;
using System.Text;
using FitConnect.Models;
using Microsoft.Extensions.Logging;

namespace FitConnect;

public class FunctionalBaseClass : EncryptionBaseClass {
    protected readonly ILogger? logger;
    public FitConnectEndpoints Endpoints { get; }

    protected FunctionalBaseClass(ILogger? logger, FitConnectEndpoints? endpoints) : base(logger) {
        Endpoints = endpoints ??
                    FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development);

        this.logger = logger;
    }


    /// <summary>
    /// Requesting an OAuth token from the FitConnect API.
    /// 
    /// You can get the Client ID and Client Secret from the FitConnect Self Service portal
    /// under https://portal.auth-testing.fit-connect.fitko.dev
    /// </summary>
    /// <param name="clientId">Your client Id</param>
    /// <param name="clientSecret">Your client Secret</param>
    /// <param name="scope">Scope if needed</param>
    /// <returns></returns>
    public async Task<OAuthAccessToken?> GetTokenAsync(string clientId, string clientSecret,
        string? scope = null) {
        var client = new HttpClient();
        client.DefaultRequestHeaders.Accept.Add(
            MediaTypeWithQualityHeaderValue.Parse("application/json"));

        var requestContent = new Dictionary<string, string> {
            { "grant_type", "client_credentials" },
            { "client_id", clientId },
            { "client_secret", clientSecret }
        };

        if (scope != null)
            requestContent["scope"] = scope;

        var content = new FormUrlEncodedContent(requestContent);

        var request = new HttpRequestMessage(HttpMethod.Post, Endpoints.TokenUrl) {
            Content = content,
            Method = HttpMethod.Post
        };

        var response = await client.SendAsync(request);

        return await response.Content.ReadFromJsonAsync<OAuthAccessToken>();
    }

    /// <summary>
    /// Receive the SET data
    /// </summary>
    /// <returns></returns>
    public Task<SecurityEventToken> GetSetDataAsync() {
        throw new NotImplementedException();
    }


    private async Task<T?> RestCall<T>(Uri uri, HttpMethod method, string body) {
        var client = new HttpClient();
        client.DefaultRequestHeaders.Add("Accept", "application/json");
        client.DefaultRequestHeaders.Add("Content-Type", "application/json");

        var request = new HttpRequestMessage();
        request.Method = method;
        request.Content = new StringContent(body, Encoding.UTF8, "application/json");
        request.RequestUri = uri;

        var response = await client.SendAsync(request);

        if (response.IsSuccessStatusCode)
            return (await response.Content.ReadFromJsonAsync<T>());

        throw new HttpRequestException("Error calling FitConnect API");
    }
}
