using System.Reflection;
using FitConnect.Models;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json.Schema;

namespace FitConnect;

public class Subscriber : FunctionalBaseClass {
    public Subscriber(ILogger logger, FitConnectEndpoints endpoints) : base(logger, endpoints) {
    }


    private async Task<string> GetContentFromEmbeddedResource(string filename) {
        var assembly = typeof(Subscriber).GetTypeInfo().Assembly;
        var resourceName = $"{assembly.GetName().Name}.{filename}";
        using (var stream = assembly.GetManifestResourceStream(resourceName)) {
            if (stream == null) {
                throw new Exception($"Could not find embedded resource {resourceName}");
            }

            using (var reader = new StreamReader(stream)) {
                return await reader.ReadToEndAsync();
            }
        }
    }

    /// <summary>
    /// Decrypt Data (Fachdaten)
    /// </summary>
    /// <param name="data">(Fachdaten)</param>
    /// <returns></returns>
    public async Task<byte[]> DecryptDataAsync(byte[] data) {
        throw new NotImplementedException();
    }

    /// <summary>
    /// Decrypt attachments (Anhänge)
    /// </summary>
    /// <param name="attachment">Encrypted attachments</param>
    /// <returns></returns>
    public async Task<byte[]> DecryptAttachmentAsync(byte[] attachment) {
        throw new NotImplementedException();
    }

    /// <summary>
    /// Checks the validity of the given metadata against the schema.
    /// </summary>
    /// <param name="jsonMetaData">JSON meta data</param>
    /// <returns></returns>
    public async Task<bool> CheckMetadataAsync(string jsonMetaData) {
        var jsonScheme =
            JSchema.Parse(await GetContentFromEmbeddedResource("metadata.schema.json"));

        var meta = JObject.Parse(jsonMetaData);
        return meta.IsValid(jsonScheme);
    }


    /// <summary>
    /// Check Hash from Metadata
    /// </summary>
    /// <param name="jsonMetaData">Metadata in JSON Format</param>
    /// <returns></returns>
    public async Task<bool> CheckHashAsync(string jsonMetaData) {
        throw new NotImplementedException();
    }


    /// <summary>
    /// Create SecurityEventToken and signature
    /// </summary>
    /// <param name="data"></param>
    /// <param name="attachment"></param>
    /// <param name="privateKey"></param>
    /// <returns></returns>
    public async Task<SecurityEventToken> CreateSecurityEventTokenAsync(string data,
        string attachment, string privateKey) {
        throw new NotImplementedException();
    }
}
